function hexPart(octal) {
    return octal.toString(16).padStart(2, '0');
}
export function rgbToHex({ r, g, b, a }) {
    const array = a ? [r, g, b, a] : [r, g, b];
    return '#' + array.map(octal => hexPart(octal)).join('');
}
export function hexToRgb({ hex, a, returnType }) {
    hex = hex.replace('#', '');
    const bigInt = parseInt(hex, 16);
    const r = (bigInt >> 16) & 255;
    const g = (bigInt >> 8) & 255;
    const b = bigInt & 255;
    return formatRGB({ r, g, b, returnType });
}
export function formatRGB({ r, g, b, a, returnType = 'obj' }) {
    if (returnType === 'html')
        return a ? `rgba(${r},${g},${b},${a})` : `rgb(${r},${g},${b})`;
    if (returnType === 'array')
        return a ? [r, g, b, a] : [r, g, b];
    if (returnType === 'obj')
        return a ? { r: r, g: g, b: b, a: a } : { r: r, g: g, b: b };
}
export function colorShift(color, amount) {
    if (!color)
        return;
    const hex = '#' +
        color
            .replace(/^#/, '')
            .replace(/../g, color => ('0' + Math.min(255, Math.max(0, parseInt(color, 16) + amount)).toString(16)).substring(-2));
    return hex;
}
export const getComplementaryColor = (hexColor) => {
    if (!hexColor)
        return hexColor;
    const colorPart = hexColor.slice(1); // remove hashtag
    const colorInteger = parseInt(colorPart, 16); // turns hex color into a integer from 0 to ~16.7 million
    let hex = ((1 << (4 * colorPart.length)) - 1 - colorInteger).toString(16); // Find compliment & convert to hex string
    while (hex.length < colorPart.length) {
        hex = '0' + hex; // zerofill
    }
    return '#' + hex;
};
export const blackOrWhite = (hexColor) => {
    if (!hexColor)
        return;
    // console.log('blackOrWhite', hexColor)
    const { r, g, b } = hexToRgb({ hex: hexColor, returnType: 'obj' });
    // let rgbNums = { r: 0, b: 0, g: 0 }
    let light = colourIsLight({ r: r, g: g, b: b });
    if (light) {
        return '#000000';
    }
    else {
        return '#ffffff';
    }
};
export const colourIsLight = ({ r, g, b }) => {
    // Counting the perceptive luminance
    // human eye favors green color...
    // console.log('colorIsLight:', r, g, b)
    const a = 1 - (0.299 * r + 0.587 * g + 0.114 * b) / 255;
    // console.log(a)
    return a < 0.5;
};
export const randomColor = (returnType = 'hex') => {
    const r = Math.floor(Math.random() * 256);
    const g = Math.floor(Math.random() * 256);
    const b = Math.floor(Math.random() * 256);
    if (returnType === 'hex')
        return rgbToHex({ r, g, b });
    return formatRGB({ r, g, b });
};
export const randomHex = () => {
    return randomColor('hex');
};
